@@ -1,8 +1,8 @@
 /*
  * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
  *
- * Copyright (c) 2013-2014 Oracle and/or its affiliates. All rights reserved.
- * Copyright (c) 2013-2014 Jason Mehrens. All rights reserved.
+ * Copyright (c) 2013-2015 Oracle and/or its affiliates. All rights reserved.
+ * Copyright (c) 2013-2015 Jason Mehrens. All rights reserved.
  *
  * The contents of this file are subject to the terms of either the GNU
  * General Public License Version 2 only ("GPL") or the Common Development
@@ -40,24 +40,25 @@
  */
 package com.sun.mail.util.logging;
 
-import java.util.Collections;
-import java.util.Date;
-import java.util.Locale;
-import java.util.ResourceBundle;
-import java.util.logging.LogManager;
+import java.util.*;
 import java.util.logging.LogRecord;
 
 /**
  * A plain text formatter that can produce fixed width output. By default this
- * formatter will produce output no greater than 160 characters wide. Only
- * specified fields support an {@link #toAlternate(java.lang.String) alternate}
- * fixed width format.
+ * formatter will produce output no greater than 160 characters wide plus the
+ * separator and newline characters. Only specified fields support an
+ * {@linkplain #toAlternate(java.lang.String) alternate} fixed width format.
  * <p>
- * The LogManager properties are:
+ * By default each <tt>CompactFormatter</tt> is initialized using the
+ * following LogManager configuration properties where
+ * <tt>&lt;formatter-name&gt;</tt> refers to the fully qualified class name or
+ * the fully qualified derived class name of the formatter.  If properties are
+ * not defined, or contain invalid values, then the specified default values are
+ * used.
  * <ul>
- * <li>&lt;formatter-name&gt;.format - the {@link java.util.Formatter
- *     format} string used to transform the output. The format string can be used to
- * fix the output size. (defaults to "%7$#.160s%n")</li>
+ * <li>&lt;formatter-name&gt;.format - the {@linkplain java.util.Formatter
+ *     format} string used to transform the output. The format string can be
+ * used to fix the output size. (defaults to <tt>%7$#.160s%n</tt>)</li>
  * </ul>
  *
  * @author Jason Mehrens
@@ -81,9 +82,9 @@ public class CompactFormatter extends java.util.logging.Formatter {
     /**
      * Creates an instance with the given format pattern.
      *
-     * @param format the {@link java.util.Formatter pattern} or null to use the
-     * LogManager default. The arguments are described in the
-     * {@link #format(java.util.logging.LogRecord) format} method.
+     * @param format the {@linkplain java.util.Formatter pattern} or null to use
+     * the LogManager default. The arguments are described in the
+     * {@linkplain #format(java.util.logging.LogRecord) format} method.
      */
     public CompactFormatter(final String format) {
         String p = getClass().getName();
@@ -92,36 +93,114 @@ public class CompactFormatter extends java.util.logging.Formatter {
 
     /**
      * Format the given log record and returns the formatted string. The
-     * {@link java.util.Formatter#format(java.lang.String, java.lang.Object...)
+     * {@linkplain java.util.Formatter#format(java.lang.String, java.lang.Object...)
      * java.util} argument indexes are assigned to the following properties:
      *
      * <ol start='0'>
-     * <li>{@code format} - the {@link java.util.Formatter
+     * <li>{@code format} - the {@linkplain java.util.Formatter
      *     java.util.Formatter} format string specified in the
      * &lt;formatter-name&gt;.format property or the format that was given when
      * this formatter was created.</li>
-     * <li>{@code date} - a {@link Date} object representing
+     * <li>{@code date} - a {@linkplain Date} object representing
      * {@linkplain LogRecord#getMillis event time} of the log record.</li>
      * <li>{@code source} - a string representing the caller, if available;
      * otherwise, the logger's name.</li>
-     * <li>{@code logger} - the logger's name.</li>
+     * <li>{@code logger} - the logger's
+     * {@linkplain LogRecord#getLoggerName() name}.</li>
      * <li>{@code level} - the
      * {@linkplain java.util.logging.Level#getLocalizedName log level}.</li>
      * <li>{@code message} - the formatted log message returned from the
-     * {@link #formatMessage(LogRecord)} method.</li>
+     * {@linkplain #formatMessage(LogRecord)} method.</li>
      * <li>{@code thrown} - a string representing the
      * {@linkplain LogRecord#getThrown throwable} associated with the log record
-     * and a relevant stack trace element if available. Otherwise, an empty
+     * and a relevant stack trace element if available; otherwise, an empty
      * string is used.</li>
      * <li>{@code message|thrown} The message and the thrown properties joined
      * as one parameter. This parameter supports
-     * {@link #toAlternate(java.lang.String) alternate} form.</li>
+     * {@linkplain #toAlternate(java.lang.String) alternate} form.</li>
      * <li>{@code thrown|message} The thrown and message properties joined as
      * one parameter. This parameter supports
-     * {@link #toAlternate(java.lang.String) alternate} form.</li>
+     * {@linkplain #toAlternate(java.lang.String) alternate} form.</li>
+     * <li>{@code sequence} the
+     * {@linkplain LogRecord#getSequenceNumber() sequence number} if the given
+     * log record.</li>
+     * <li>{@code thread id} the {@linkplain LogRecord#getThreadID() thread id}
+     * of the given log record.  By default this is formatted as a {@code long}
+     * by an unsigned conversion.</li>
+     * <li>{@code error} the throwable simple class name and
+     * {@linkplain #formatError(LogRecord) error message} without any
+     * stack trace.</li>
+     * <li>{@code message|error} The message and error properties joined as
+     * one parameter. This parameter supports
+     * {@linkplain #toAlternate(java.lang.String) alternate} form.</li>
+     * <li>{@code error|message} The error and message properties joined as
+     * one parameter. This parameter supports
+     * {@linkplain #toAlternate(java.lang.String) alternate} form.</li>
+     * <li>{@code backtrace} only the
+     * {@linkplain #formatBackTrace(LogRecord) stack trace} of the given
+     * throwable.</li>
+     * <li>{@code bundlename} the resource bundle
+     * {@linkplain LogRecord#getResourceBundleName() name} of the given log
+     * record.</li>
+     * <li>{@code key} the {@linkplain LogRecord#getMessage() raw message}
+     * before localization or formatting.</li>
      * </ol>
      *
-     * @param record to format.
+     * <p>
+     * Some example formats:<br>
+     * <ul>
+     * <li>{@code com.sun.mail.util.logging.CompactFormatter.format=%7$#.160s%n}
+     * <p>
+     * This prints only 160 characters of the message|thrown ({@code 7$}) using
+     * the {@linkplain #toAlternate(java.lang.String) alternate} form. The
+     * separator is not included as part of the total width.
+     * <pre>
+     * Encoding failed.|NullPointerException: null String.getBytes(:913)
+     * </pre>
+     *
+     * <li>{@code com.sun.mail.util.logging.CompactFormatter.format=%7$#.20s%n}
+     * <p>
+     * This prints only 20 characters of the message|thrown ({@code 7$}) using
+     * the {@linkplain #toAlternate(java.lang.String) alternate} form.  This
+     * will perform a weighted truncation of both the message and thrown
+     * properties of the log record.  The separator is not included as part of
+     * the total width.
+     * <pre>
+     * Encoding|NullPointerE
+     * </pre>
+     *
+     * <li>{@code com.sun.mail.util.logging.CompactFormatter.format=%1$tc %2$s%n%4$s: %5$s%6$s%n}
+     * <p>
+     * This prints the timestamp ({@code 1$}) and the source ({@code 2$}) on the
+     * first line. The second line is the log level ({@code 4$}), log message
+     * ({@code 5$}), and the throwable with a relevant stack trace element
+     * ({@code 6$}) if one is available.
+     * <pre>
+     * Fri Nov 20 07:29:24 CST 2009 MyClass fatal
+     * SEVERE: Encoding failed.NullPointerException: null String.getBytes(:913)
+     * </pre>
+     *
+     * <li>{@code com.sun.mail.util.logging.CompactFormatter.format=%4$s: %12$#.160s%n}
+     * <p>
+     * This prints the log level ({@code 4$}) and only 160 characters of the
+     * message|error ({@code 12$}) using the alternate form.
+     * <pre>
+     * SEVERE: Unable to send notification.|SocketException: Permission denied: connect
+     * </pre>
+     *
+     * <li>{@code com.sun.mail.util.logging.CompactFormatter.format=[%9$d][%1$tT][%10$d][%2$s] %5$s%n%6$s%n}
+     * <p>
+     * This prints the sequence ({@code 9$}), event time ({@code 1$}) as
+     * 24 hour time, thread id ({@code 10$}), source ({@code 2$}), log message
+     * ({@code 5$}), and the throwable with back trace ({@code 6$}).
+     * <pre>
+     * [125][14:11:42][38][MyClass fatal] Unable to send notification.
+     * SocketException: Permission denied: connect SMTPTransport.openServer(:1949)
+     * </pre>
+     *
+     * </ul>
+     *
+     * @param record the log record to format.
      * @return the formatted record.
      * @throws NullPointerException if the given record is null.
      */
@@ -133,6 +212,7 @@ public class CompactFormatter extends java.util.logging.Formatter {
 
         String msg = formatMessage(record);
         String thrown = formatThrown(record);
+        String err = formatError(record);
         Object[] params = {
             new Date(record.getMillis()),
             formatSource(record),
@@ -141,9 +221,21 @@ public class CompactFormatter extends java.util.logging.Formatter {
             msg,
             thrown,
             new Alternate(msg, thrown),
-            new Alternate(thrown, msg)};
+            new Alternate(thrown, msg),
+            record.getSequenceNumber(),
+            formatThreadID(record),
+            err,
+            new Alternate(msg, err),
+            new Alternate(err, msg),
+            formatBackTrace(record),
+            record.getResourceBundleName(),
+            record.getMessage()};
 
-        return String.format(l, fmt, params);
+        if (l == null) { //BUG ID 6282094
+            return String.format(fmt, params);
+        } else {
+            return String.format(l, fmt, params);
+        }
     }
 
     /**
@@ -166,8 +258,9 @@ public class CompactFormatter extends java.util.logging.Formatter {
      * method removes any fully qualified throwable class names from the message
      * cause chain.
      *
-     * @param t the throwable to format.
-     * @return the formatted message string from the throwable.
+     * @param t the throwable to format or null.
+     * @return the empty string if null was given or the formatted message
+     * string from the throwable which may be null.
      */
     public String formatMessage(final Throwable t) {
         return t != null ? replaceClassName(apply(t).getMessage(), t) : "";
@@ -217,6 +310,24 @@ public class CompactFormatter extends java.util.logging.Formatter {
         return simpleClassName(record.getLoggerName());
     }
 
+    /**
+     * Formats the thread id property of the given log record.  By default this
+     * is formatted as a {@code long} by an unsigned conversion.
+     *
+     * @param record the record.
+     * @return the formatted thread id as a number.
+     * @throws NullPointerException if the given record is null.
+     * @since JavaMail 1.5.4
+     */
+    public Number formatThreadID(final LogRecord record) {
+        /**
+         * Thread.getID is defined as long and LogRecord.getThreadID is defined
+         * as int.  Convert to unsigned as a means to better map the two types
+         * of thread identifiers.
+         */
+        return (((long) record.getThreadID()) & 0xffffffffL);
+    }
+
     /**
      * Formats the thrown property of a LogRecord. The returned string will
      * contain a throwable message with a back trace.
@@ -231,21 +342,48 @@ public class CompactFormatter extends java.util.logging.Formatter {
         String msg;
         final Throwable t = record.getThrown();
         if (t != null) {
-            final Throwable root = apply(t);
-            if (root != null) {
-                msg = formatMessage(t);
-                String site = formatBackTrace(record);
-                msg = root.getClass().getSimpleName() + ": " + msg
-                        + (isNullOrSpaces(site) ? "" : ' ' + site);
-            } else {
-                msg = "";
-            }
+            String site = formatBackTrace(record);
+            msg = formatToString(t) + (isNullOrSpaces(site) ? "" : ' ' + site);
         } else {
             msg = "";
         }
         return msg;
     }
 
+    /**
+     * Formats the thrown property of a LogRecord as an error message. The
+     * returned string will not contain a back trace.
+     *
+     * @param record the record.
+     * @return empty string if nothing was thrown or formatted string.
+     * @throws NullPointerException if the given record is null.
+     * @see Throwable#toString()
+     * @see #apply(java.lang.Throwable)
+     * @see #formatMessage(java.lang.Throwable)
+     * @since JavaMail 1.5.4
+     */
+    public String formatError(final LogRecord record) {
+        Throwable t = record.getThrown();
+        if (t != null) {
+            return formatToString(t);
+        } else {
+            return "";
+        }
+    }
+
+    /**
+     * Gets the simple class name of the reduced throwable and message of
+     * the reduced throwable.
+     *
+     * @param t the given throwable.
+     * @return the formatted throwable.
+     * @since JavaMail 1.5.4
+     * @throws NullPointerException if given throwable is null.
+     */
+    private String formatToString(Throwable t) {
+        return simpleClassName(apply(t).getClass()) + ": " + formatMessage(t);
+    }
+
     /**
      * Formats the back trace for the given log record.
      *
@@ -261,21 +399,19 @@ public class CompactFormatter extends java.util.logging.Formatter {
         final Throwable t = record.getThrown();
         if (t != null) {
             final Throwable root = apply(t);
-            if (root != null) {
-                site = findAndFormat(root.getStackTrace());
-                if (isNullOrSpaces(site)) {
-                    int limit = 0;
-                    for (Throwable c = t; c != null; c = c.getCause()) {
-                        site = findAndFormat(c.getStackTrace());
-                        if (!isNullOrSpaces(site)) {
-                            break;
-                        }
+            site = findAndFormat(root.getStackTrace());
+            if (isNullOrSpaces(site)) {
+                int limit = 0;
+                for (Throwable c = t; c != null; c = c.getCause()) {
+                    site = findAndFormat(c.getStackTrace());
+                    if (!isNullOrSpaces(site)) {
+                        break;
+                    }
 
-                        //Deal with excessive cause chains
-                        //and cyclic throwables.
-                        if (++limit == (1 << 16)) {
-                            break; //Give up.
-                        }
+                    //Deal with excessive cause chains
+                    //and cyclic throwables.
+                    if (++limit == (1 << 16)) {
+                        break; //Give up.
                     }
                 }
             }
@@ -343,7 +479,7 @@ public class CompactFormatter extends java.util.logging.Formatter {
      * formatting.
      *
      * @param t the throwable from the log record.
-     * @return the throwable or null.
+     * @return the chosen throwable or null only if the given argument is null.
      * @see #formatThrown(java.util.logging.LogRecord)
      */
     protected Throwable apply(final Throwable t) {
@@ -362,7 +498,7 @@ public class CompactFormatter extends java.util.logging.Formatter {
     }
 
     /**
-     * Defines the alternate format. This implementation removes all control
+     * Defines the alternate format.  This implementation removes all control
      * characters from the given string.
      *
      * @param s any string or null.
@@ -397,9 +533,9 @@ public class CompactFormatter extends java.util.logging.Formatter {
         } catch (Exception ignore) {
         } catch (LinkageError ignore) {
         }
-        return (!s.getClassName().endsWith("es")
-                && s.getClassName().endsWith("s"))
-                || s.getClassName().contains("Util");
+        final String cn = s.getClassName();
+        return (cn.endsWith("s") && !cn.endsWith("es"))
+                || cn.contains("Util") || cn.endsWith("Throwables");
     }
 
     /**
@@ -451,8 +587,7 @@ public class CompactFormatter extends java.util.logging.Formatter {
      * @throws NullPointerException if the given class name is null.
      */
     private String initFormat(final String p) {
-        LogManager m = LogManagerProperties.getLogManager();
-        String v = m.getProperty(p.concat(".format"));
+        String v = LogManagerProperties.fromLogManager(p.concat(".format"));
         if (isNullOrSpaces(v)) {
             v = "%7$#.160s%n"; //160 chars split between message and thrown.
         }
@@ -473,7 +608,7 @@ public class CompactFormatter extends java.util.logging.Formatter {
             int limit = 0;
             for (Throwable c = t; c != null; c = c.getCause()) {
                 final Class<?> k = c.getClass();
-                msg = msg.replace(k.getName(), k.getSimpleName());
+                msg = msg.replace(k.getName(), simpleClassName(k));
 
                 //Deal with excessive cause chains and cyclic throwables.
                 if (++limit == (1 << 16)) {
@@ -498,13 +633,29 @@ public class CompactFormatter extends java.util.logging.Formatter {
             for (Object o : p) {
                 if (o != null) {
                     final Class<?> k = o.getClass();
-                    msg = msg.replace(k.getName(), k.getSimpleName());
+                    msg = msg.replace(k.getName(), simpleClassName(k));
                 }
             }
         }
         return msg;
     }
 
+    /**
+     * Gets the simple class name from the given class. This is a workaround for
+     * BUG ID JDK-8057919.
+     *
+     * @param k the class object.
+     * @return the simple class name or null.
+     * @since JavaMail 1.5.3
+     */
+    private static String simpleClassName(final Class<?> k) {
+        try {
+            return k.getSimpleName();
+        } catch (final InternalError JDK8057919) {
+        }
+        return simpleClassName(k.getName());
+    }
+
     /**
      * Converts a fully qualified class name to a simple class name.
      *
