@@ -298,7 +298,7 @@ public class MimeBodyPart extends BodyPart implements MimePart {
     }
 
     /**
-     * Returns the value of the "Content-Disposition" header field.
+     * Returns the disposition from the "Content-Disposition" header field.
      * This represents the disposition of this part. The disposition
      * describes how the part should be presented to the user. <p>
      *
@@ -316,9 +316,9 @@ public class MimeBodyPart extends BodyPart implements MimePart {
     }
 
     /**
-     * Set the "Content-Disposition" header field of this body part.
-     * If the disposition is null, any existing "Content-Disposition"
-     * header field is removed.
+     * Set the disposition in the "Content-Disposition" header field
+     * of this body part.  If the disposition is null, any existing
+     * "Content-Disposition" header field is removed.
      *
      * @exception	IllegalWriteException if the underlying
      *			implementation does not support modification
@@ -1033,6 +1033,7 @@ public class MimeBodyPart extends BodyPart implements MimePart {
      * Return all the headers from this Message as an Enumeration of
      * Header objects.
      */
+    @SuppressWarnings("rawtypes")
     public Enumeration getAllHeaders() throws MessagingException {
 	return headers.getAllHeaders();
     }
@@ -1041,6 +1042,7 @@ public class MimeBodyPart extends BodyPart implements MimePart {
      * Return matching headers from this Message as an Enumeration of
      * Header objects.
      */
+    @SuppressWarnings("rawtypes")
     public Enumeration getMatchingHeaders(String[] names)
                         throws MessagingException {
 	return headers.getMatchingHeaders(names);
@@ -1050,6 +1052,7 @@ public class MimeBodyPart extends BodyPart implements MimePart {
      * Return non-matching headers from this Message as an
      * Enumeration of Header objects.
      */
+    @SuppressWarnings("rawtypes")
     public Enumeration getNonMatchingHeaders(String[] names)
                         throws MessagingException {
 	return headers.getNonMatchingHeaders(names);
@@ -1067,6 +1070,7 @@ public class MimeBodyPart extends BodyPart implements MimePart {
      * line is a raw RFC 822 header line, containing both the "name"
      * and "value" field.
      */
+    @SuppressWarnings("rawtypes")
     public Enumeration getAllHeaderLines() throws MessagingException {
   	return headers.getAllHeaderLines(); 
     }
@@ -1076,6 +1080,7 @@ public class MimeBodyPart extends BodyPart implements MimePart {
      * A Header line is a raw RFC 822 header line, containing both
      * the "name" and "value" field.
      */
+    @SuppressWarnings("rawtypes")
     public Enumeration getMatchingHeaderLines(String[] names)
                                     throws MessagingException {
 	return headers.getMatchingHeaderLines(names);
@@ -1086,6 +1091,7 @@ public class MimeBodyPart extends BodyPart implements MimePart {
      * A Header line is a raw RFC 822 header line, containing both
      * the "name"  and "value" field.
      */
+    @SuppressWarnings("rawtypes")
     public Enumeration getNonMatchingHeaderLines(String[] names)  
                                         throws MessagingException {
 	return headers.getNonMatchingHeaderLines(names);
@@ -1282,7 +1288,10 @@ public class MimeBodyPart extends BodyPart implements MimePart {
 	    p = new ParameterList();
 	    cd.setParameterList(p);
 	}
-	p.set("filename", name, charset);
+	if (encodeFileName)
+	    p.setLiteral("filename", name);
+	else
+	    p.set("filename", name, charset);
 	part.setHeader("Content-Disposition", cd.toString());
 
 	/*
@@ -1301,7 +1310,10 @@ public class MimeBodyPart extends BodyPart implements MimePart {
 			p = new ParameterList();
 			cType.setParameterList(p);
 		    }
-		    p.set("name", name, charset);
+		    if (encodeFileName)
+			p.setLiteral("name", name);
+		    else
+			p.set("name", name, charset);
 		    part.setHeader("Content-Type", cType.toString());
 		} catch (ParseException pex) { }	// ignore it
 	    }
@@ -1317,7 +1329,7 @@ public class MimeBodyPart extends BodyPart implements MimePart {
 
 	// Tokenize the header to obtain the Language-tags (skip comments)
 	HeaderTokenizer h = new HeaderTokenizer(s, HeaderTokenizer.MIME);
-	Vector v = new Vector();
+	List<String> v = new ArrayList<String>();
 
 	HeaderTokenizer.Token tk;
 	int tkType;
@@ -1328,16 +1340,16 @@ public class MimeBodyPart extends BodyPart implements MimePart {
 	    if (tkType == HeaderTokenizer.Token.EOF)
 		break; // done
 	    else if (tkType == HeaderTokenizer.Token.ATOM)
-		v.addElement(tk.getValue());
+		v.add(tk.getValue());
 	    else // invalid token, skip it.
 		continue;
 	}
 
-	if (v.size() == 0)
+	if (v.isEmpty())
 	    return null;
 
 	String[] language = new String[v.size()];
-	v.copyInto(language);
+	v.toArray(language);
 	return language;	
     }
 
@@ -1558,7 +1570,11 @@ public class MimeBodyPart extends BodyPart implements MimePart {
 				p = new ParameterList();
 				cType.setParameterList(p);
 			    }
-			    p.set("name", filename,
+			    if (encodeFileName)
+				p.setLiteral("name",
+					MimeUtility.encodeText(filename));
+			    else
+				p.set("name", filename,
 					MimeUtility.getDefaultMIMECharset());
 			    type = cType.toString();
 			}
@@ -1590,9 +1606,11 @@ public class MimeBodyPart extends BodyPart implements MimePart {
 	}
 
 	// First, write out the header
-	Enumeration hdrLines = part.getNonMatchingHeaderLines(ignoreList);
+	@SuppressWarnings("unchecked")
+	Enumeration<String> hdrLines
+		= part.getNonMatchingHeaderLines(ignoreList);
 	while (hdrLines.hasMoreElements())
-	    los.writeln((String)hdrLines.nextElement());
+	    los.writeln(hdrLines.nextElement());
 
 	// The CRLF separator between header and content
 	los.writeln();
