@@ -62,7 +62,7 @@ public final class IMAPIdleStateTest {
 
     // timeout the test in case of deadlock
     @Rule
-    public Timeout deadlockTimeout = new Timeout(20000);
+    public Timeout deadlockTimeout = Timeout.seconds(20);
 
     @Test
     public void test() {
