@@ -65,7 +65,7 @@ public final class IMAPIdleUntaggedResponseTest {
 
     // timeout the test in case of deadlock
     @Rule
-    public Timeout deadlockTimeout = new Timeout(20000);
+    public Timeout deadlockTimeout = Timeout.seconds(20);
 
     @Test
     public void test() {
